#!/usr/bin/env python
#

"""Generate thumbnails for comics files using comicthumb
"""
import os, sys
import md5
import findrox ; findrox.version ( 2, 0, 2)
import rox, rox.mime
import pango
import thumb
import zipfile
import tarfile
import StringIO
import re
import locale
import shutil
import Image

def binaryInPath(b):
	for path in os.environ["PATH"].split(":"):
		if os.access(os.path.join(path, b), os.R_OK | os.X_OK):
			return True
	return False

def execute_return_err(cmd):
    errmsg=""
    cin, cout=os.popen4(cmd, 'r')
    cin.close()
    for l in cout:
        errmsg+=l
    cout.close()
    #cerr.close()
    if errmsg:
        return errmsg

def firstImage (filelist):
	exts=re.compile('jpg$|png$|jpeg$|gif$', re.IGNORECASE)
	for file in filelist:
		if exts.search(file):
			return file
	return False

# try to find the cover, if nothing good is found take first
def guessCover (filelist):
	# very basic, should not get eg backcover if possible
	p = re.compile('cover|front', re.IGNORECASE)
	coverlist = filter(p.search, filelist)
	firstcover=firstImage(coverlist)
	if firstcover:
		return firstcover 
	else:
		return firstImage(filelist)

def CreateThumbnail ( input, output, size ):
	"""based on comicthumb from comix"""
	locale.setlocale(locale.LC_COLLATE, os.getenv('LC_COLLATE'))
	in_file_path = input
	out_file_path = output
	THUMB_SIZE = int(size)
	# still need a temporary dir for rars currently
	if not os.path.exists('/tmp/comicthumb/'):
		os.makedirs('/tmp/comicthumb/')
		os.chmod('/tmp/comicthumb/', 0700)
	try:
		if zipfile.is_zipfile(in_file_path):
			type = 'cbz'
			zip = zipfile.ZipFile(in_file_path, "r")
			zipfiles = zip.namelist()
			zipfiles.sort(locale.strcoll)
			cover=guessCover(zipfiles)
			image=Image.open(StringIO.StringIO(zip.read(cover)))
			zip.close()
		elif tarfile.is_tarfile(in_file_path):
			type = 'cbt'
			file = open(in_file_path, 'rb')
			tar = tarfile.open(in_file_path, "r")
			tarfiles = tar.getnames()
			tarfiles.sort(locale.strcoll)
			cover=guessCover(tarfiles)
			picture=StringIO.StringIO(tar.extractfile(cover).read())
			image=Image.open(picture)
			tar.close()
		elif open(in_file_path, 'rb').read(4) == 'Rar!':
			type = 'cbr'
			if os.popen("unrar h").close():
				print "You need to install unrar to thumbnail rar files."
				sys.exit(1)
			rarfiles=os.popen('unrar vb "' + in_file_path + '"').readlines()
			for i in range(len(rarfiles)):
				rarfiles[i]=rarfiles[i].rstrip("\n")
			rarfiles.sort(locale.strcoll)
			cover = guessCover(rarfiles)
			os.popen('unrar p -inul "' + in_file_path + '" "' + cover + '" > /tmp/comicthumb/temp.jpg', "r").close()
			image=Image.open("/tmp/comicthumb/temp.jpg")
		
		if image.size[0] > image.size[1]:
			x = THUMB_SIZE
			y = THUMB_SIZE * image.size[1] / image.size[0]
		else:
			x = THUMB_SIZE * image.size[0] / image.size[1]
			y = THUMB_SIZE
		image = image.resize((x, y), Image.ANTIALIAS)
		image = image.convert('RGB')
		
		try:
			if os.path.exists('../share/pixmaps/comix/' + type + '.png'):
				overlay_path = '../share/pixmaps/comix/' + type + '.png'
			elif os.path.exists('/usr/local/share/pixmaps/comix/' + type + '.png'):
				overlay_path = '/usr/local/share/pixmaps/comix/' + type + '.png'
			elif os.path.exists('/usr/share/pixmaps/comix/' + type + '.png'):
				overlay_path = '/usr/share/pixmaps/comix/' + type + '.png'
			
			overlay_image=Image.open(overlay_path)
			image_canvas = Image.new('RGBA', (image.size[0], image.size[1]), (0, 0, 0, 0))
			image_canvas.paste(overlay_image, (image.size[0] - 35, image.size[1] - 30))
			image = Image.composite(image_canvas, image, image_canvas)
		except:
			pass
	
		image.save(out_file_path, 'PNG')
	except:
		pass

	if os.path.isdir('/tmp/comicthumb/'):
		shutil.rmtree('/tmp/comicthumb/')

class ComicThumbNail(thumb.Thumbnailler):
	"""Generate thumbnail for comix files"""
	def __init__(self, debug=False):
		"""Initialize Comix thumbnailler"""
		thumb.Thumbnailler.__init__(self, 'ComicThumbNail', 'comic',True, debug)
	
	def failed_image(self, rsize, tstr):
		if debug: print 'failed_image', self, rsize, tstr
		if not tstr:
			tstr='Error!'
		w=rsize
		h=rsize/4*3
		try:
			p=rox.g.gdk.Pixbuf(rox.g.gdk.COLORSPACE_RGB, False, 8, w, h)
		except:
			sys.exit(2)
		pixmap, mask=p.render_pixmap_and_mask()
		cmap=pixmap.get_colormap()
		gc=pixmap.new_gc(foreground=cmap.alloc_color('black'))
		gc.set_foreground(gc.background)

		
		pixmap.draw_rectangle(gc, rox.g.TRUE, 0, 0, w, h)
        
		gc.set_foreground(cmap.alloc_color('red'))
		dummy=rox.g.Window()
		layout=dummy.create_pango_layout(tstr)
		if w>40:
			layout.set_width((w-10)*pango.SCALE)
            #layout.set_wrap(pango.WRAP_CHAR)
		pixmap.draw_layout(gc, 10, 4, layout)
		if debug: print pixmap

		self.add_time=False
        
		return p.get_from_drawable(pixmap, cmap, 0, 0, 0, 0, -1, -1)
	def get_image(self, inname, rsize):
		outfile = os.path.join(self.work_dir, "out.png")

		#cmd = 'comicthumb \"%s\" %s' % (inname, outfile)

		#errmsg=execute_return_err(cmd)
		CreateThumbnail( inname , outfile , rsize )
		if not os.path.exists(outfile):
			return self.failed_image(rsize, 'error')

        # Now we load the raw image in
		return rox.g.gdk.pixbuf_new_from_file(outfile)
     
def main (argv):
	inname = argv [0]
	outname = argv [1]
	rsize = argv [2]
	thumb = ComicThumbNail(0)
	thumb.run(inname, outname, rsize)

